# Copyright 2023 The KServe Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# coding: utf-8

"""
    KServe

    Python SDK for KServe  # noqa: E501

    The version of the OpenAPI document: v0.1
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from kserve.configuration import Configuration


class V1beta1IngressConfig(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'additional_ingress_domains': 'list[str]',
        'disable_ingress_creation': 'bool',
        'disable_istio_virtual_host': 'bool',
        'domain_template': 'str',
        'enable_gateway_api': 'bool',
        'ingress_class_name': 'str',
        'ingress_domain': 'str',
        'ingress_gateway': 'str',
        'knative_local_gateway_service': 'str',
        'kserve_ingress_gateway': 'str',
        'local_gateway': 'str',
        'local_gateway_service': 'str',
        'path_template': 'str',
        'url_scheme': 'str'
    }

    attribute_map = {
        'additional_ingress_domains': 'additionalIngressDomains',
        'disable_ingress_creation': 'disableIngressCreation',
        'disable_istio_virtual_host': 'disableIstioVirtualHost',
        'domain_template': 'domainTemplate',
        'enable_gateway_api': 'enableGatewayApi',
        'ingress_class_name': 'ingressClassName',
        'ingress_domain': 'ingressDomain',
        'ingress_gateway': 'ingressGateway',
        'knative_local_gateway_service': 'knativeLocalGatewayService',
        'kserve_ingress_gateway': 'kserveIngressGateway',
        'local_gateway': 'localGateway',
        'local_gateway_service': 'localGatewayService',
        'path_template': 'pathTemplate',
        'url_scheme': 'urlScheme'
    }

    def __init__(self, additional_ingress_domains=None, disable_ingress_creation=None, disable_istio_virtual_host=None, domain_template=None, enable_gateway_api=None, ingress_class_name=None, ingress_domain=None, ingress_gateway=None, knative_local_gateway_service=None, kserve_ingress_gateway=None, local_gateway=None, local_gateway_service=None, path_template=None, url_scheme=None, local_vars_configuration=None):  # noqa: E501
        """V1beta1IngressConfig - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._additional_ingress_domains = None
        self._disable_ingress_creation = None
        self._disable_istio_virtual_host = None
        self._domain_template = None
        self._enable_gateway_api = None
        self._ingress_class_name = None
        self._ingress_domain = None
        self._ingress_gateway = None
        self._knative_local_gateway_service = None
        self._kserve_ingress_gateway = None
        self._local_gateway = None
        self._local_gateway_service = None
        self._path_template = None
        self._url_scheme = None
        self.discriminator = None

        if additional_ingress_domains is not None:
            self.additional_ingress_domains = additional_ingress_domains
        if disable_ingress_creation is not None:
            self.disable_ingress_creation = disable_ingress_creation
        if disable_istio_virtual_host is not None:
            self.disable_istio_virtual_host = disable_istio_virtual_host
        if domain_template is not None:
            self.domain_template = domain_template
        if enable_gateway_api is not None:
            self.enable_gateway_api = enable_gateway_api
        if ingress_class_name is not None:
            self.ingress_class_name = ingress_class_name
        if ingress_domain is not None:
            self.ingress_domain = ingress_domain
        if ingress_gateway is not None:
            self.ingress_gateway = ingress_gateway
        if knative_local_gateway_service is not None:
            self.knative_local_gateway_service = knative_local_gateway_service
        if kserve_ingress_gateway is not None:
            self.kserve_ingress_gateway = kserve_ingress_gateway
        if local_gateway is not None:
            self.local_gateway = local_gateway
        if local_gateway_service is not None:
            self.local_gateway_service = local_gateway_service
        if path_template is not None:
            self.path_template = path_template
        if url_scheme is not None:
            self.url_scheme = url_scheme

    @property
    def additional_ingress_domains(self):
        """Gets the additional_ingress_domains of this V1beta1IngressConfig.  # noqa: E501


        :return: The additional_ingress_domains of this V1beta1IngressConfig.  # noqa: E501
        :rtype: list[str]
        """
        return self._additional_ingress_domains

    @additional_ingress_domains.setter
    def additional_ingress_domains(self, additional_ingress_domains):
        """Sets the additional_ingress_domains of this V1beta1IngressConfig.


        :param additional_ingress_domains: The additional_ingress_domains of this V1beta1IngressConfig.  # noqa: E501
        :type: list[str]
        """

        self._additional_ingress_domains = additional_ingress_domains

    @property
    def disable_ingress_creation(self):
        """Gets the disable_ingress_creation of this V1beta1IngressConfig.  # noqa: E501


        :return: The disable_ingress_creation of this V1beta1IngressConfig.  # noqa: E501
        :rtype: bool
        """
        return self._disable_ingress_creation

    @disable_ingress_creation.setter
    def disable_ingress_creation(self, disable_ingress_creation):
        """Sets the disable_ingress_creation of this V1beta1IngressConfig.


        :param disable_ingress_creation: The disable_ingress_creation of this V1beta1IngressConfig.  # noqa: E501
        :type: bool
        """

        self._disable_ingress_creation = disable_ingress_creation

    @property
    def disable_istio_virtual_host(self):
        """Gets the disable_istio_virtual_host of this V1beta1IngressConfig.  # noqa: E501


        :return: The disable_istio_virtual_host of this V1beta1IngressConfig.  # noqa: E501
        :rtype: bool
        """
        return self._disable_istio_virtual_host

    @disable_istio_virtual_host.setter
    def disable_istio_virtual_host(self, disable_istio_virtual_host):
        """Sets the disable_istio_virtual_host of this V1beta1IngressConfig.


        :param disable_istio_virtual_host: The disable_istio_virtual_host of this V1beta1IngressConfig.  # noqa: E501
        :type: bool
        """

        self._disable_istio_virtual_host = disable_istio_virtual_host

    @property
    def domain_template(self):
        """Gets the domain_template of this V1beta1IngressConfig.  # noqa: E501


        :return: The domain_template of this V1beta1IngressConfig.  # noqa: E501
        :rtype: str
        """
        return self._domain_template

    @domain_template.setter
    def domain_template(self, domain_template):
        """Sets the domain_template of this V1beta1IngressConfig.


        :param domain_template: The domain_template of this V1beta1IngressConfig.  # noqa: E501
        :type: str
        """

        self._domain_template = domain_template

    @property
    def enable_gateway_api(self):
        """Gets the enable_gateway_api of this V1beta1IngressConfig.  # noqa: E501


        :return: The enable_gateway_api of this V1beta1IngressConfig.  # noqa: E501
        :rtype: bool
        """
        return self._enable_gateway_api

    @enable_gateway_api.setter
    def enable_gateway_api(self, enable_gateway_api):
        """Sets the enable_gateway_api of this V1beta1IngressConfig.


        :param enable_gateway_api: The enable_gateway_api of this V1beta1IngressConfig.  # noqa: E501
        :type: bool
        """

        self._enable_gateway_api = enable_gateway_api

    @property
    def ingress_class_name(self):
        """Gets the ingress_class_name of this V1beta1IngressConfig.  # noqa: E501


        :return: The ingress_class_name of this V1beta1IngressConfig.  # noqa: E501
        :rtype: str
        """
        return self._ingress_class_name

    @ingress_class_name.setter
    def ingress_class_name(self, ingress_class_name):
        """Sets the ingress_class_name of this V1beta1IngressConfig.


        :param ingress_class_name: The ingress_class_name of this V1beta1IngressConfig.  # noqa: E501
        :type: str
        """

        self._ingress_class_name = ingress_class_name

    @property
    def ingress_domain(self):
        """Gets the ingress_domain of this V1beta1IngressConfig.  # noqa: E501


        :return: The ingress_domain of this V1beta1IngressConfig.  # noqa: E501
        :rtype: str
        """
        return self._ingress_domain

    @ingress_domain.setter
    def ingress_domain(self, ingress_domain):
        """Sets the ingress_domain of this V1beta1IngressConfig.


        :param ingress_domain: The ingress_domain of this V1beta1IngressConfig.  # noqa: E501
        :type: str
        """

        self._ingress_domain = ingress_domain

    @property
    def ingress_gateway(self):
        """Gets the ingress_gateway of this V1beta1IngressConfig.  # noqa: E501


        :return: The ingress_gateway of this V1beta1IngressConfig.  # noqa: E501
        :rtype: str
        """
        return self._ingress_gateway

    @ingress_gateway.setter
    def ingress_gateway(self, ingress_gateway):
        """Sets the ingress_gateway of this V1beta1IngressConfig.


        :param ingress_gateway: The ingress_gateway of this V1beta1IngressConfig.  # noqa: E501
        :type: str
        """

        self._ingress_gateway = ingress_gateway

    @property
    def knative_local_gateway_service(self):
        """Gets the knative_local_gateway_service of this V1beta1IngressConfig.  # noqa: E501


        :return: The knative_local_gateway_service of this V1beta1IngressConfig.  # noqa: E501
        :rtype: str
        """
        return self._knative_local_gateway_service

    @knative_local_gateway_service.setter
    def knative_local_gateway_service(self, knative_local_gateway_service):
        """Sets the knative_local_gateway_service of this V1beta1IngressConfig.


        :param knative_local_gateway_service: The knative_local_gateway_service of this V1beta1IngressConfig.  # noqa: E501
        :type: str
        """

        self._knative_local_gateway_service = knative_local_gateway_service

    @property
    def kserve_ingress_gateway(self):
        """Gets the kserve_ingress_gateway of this V1beta1IngressConfig.  # noqa: E501


        :return: The kserve_ingress_gateway of this V1beta1IngressConfig.  # noqa: E501
        :rtype: str
        """
        return self._kserve_ingress_gateway

    @kserve_ingress_gateway.setter
    def kserve_ingress_gateway(self, kserve_ingress_gateway):
        """Sets the kserve_ingress_gateway of this V1beta1IngressConfig.


        :param kserve_ingress_gateway: The kserve_ingress_gateway of this V1beta1IngressConfig.  # noqa: E501
        :type: str
        """

        self._kserve_ingress_gateway = kserve_ingress_gateway

    @property
    def local_gateway(self):
        """Gets the local_gateway of this V1beta1IngressConfig.  # noqa: E501


        :return: The local_gateway of this V1beta1IngressConfig.  # noqa: E501
        :rtype: str
        """
        return self._local_gateway

    @local_gateway.setter
    def local_gateway(self, local_gateway):
        """Sets the local_gateway of this V1beta1IngressConfig.


        :param local_gateway: The local_gateway of this V1beta1IngressConfig.  # noqa: E501
        :type: str
        """

        self._local_gateway = local_gateway

    @property
    def local_gateway_service(self):
        """Gets the local_gateway_service of this V1beta1IngressConfig.  # noqa: E501


        :return: The local_gateway_service of this V1beta1IngressConfig.  # noqa: E501
        :rtype: str
        """
        return self._local_gateway_service

    @local_gateway_service.setter
    def local_gateway_service(self, local_gateway_service):
        """Sets the local_gateway_service of this V1beta1IngressConfig.


        :param local_gateway_service: The local_gateway_service of this V1beta1IngressConfig.  # noqa: E501
        :type: str
        """

        self._local_gateway_service = local_gateway_service

    @property
    def path_template(self):
        """Gets the path_template of this V1beta1IngressConfig.  # noqa: E501


        :return: The path_template of this V1beta1IngressConfig.  # noqa: E501
        :rtype: str
        """
        return self._path_template

    @path_template.setter
    def path_template(self, path_template):
        """Sets the path_template of this V1beta1IngressConfig.


        :param path_template: The path_template of this V1beta1IngressConfig.  # noqa: E501
        :type: str
        """

        self._path_template = path_template

    @property
    def url_scheme(self):
        """Gets the url_scheme of this V1beta1IngressConfig.  # noqa: E501


        :return: The url_scheme of this V1beta1IngressConfig.  # noqa: E501
        :rtype: str
        """
        return self._url_scheme

    @url_scheme.setter
    def url_scheme(self, url_scheme):
        """Sets the url_scheme of this V1beta1IngressConfig.


        :param url_scheme: The url_scheme of this V1beta1IngressConfig.  # noqa: E501
        :type: str
        """

        self._url_scheme = url_scheme

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1beta1IngressConfig):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1beta1IngressConfig):
            return True

        return self.to_dict() != other.to_dict()
